@@ -13,6 +13,8 @@
 
 #include <config.h>
 
+#include <stdbool.h>
+
 #include <isc/magic.h>
 #include <isc/mem.h>
 #include <isc/netaddr.h>
@@ -36,13 +38,14 @@
 
 struct dns_ssurule {
 	unsigned int magic;
-	isc_boolean_t grant;	/*%< is this a grant or a deny? */
-	unsigned int matchtype;	/*%< which type of pattern match? */
-	dns_name_t *identity;	/*%< the identity to match */
-	dns_name_t *name;	/*%< the name being updated */
-	unsigned int ntypes;	/*%< number of data types covered */
-	dns_rdatatype_t *types;	/*%< the data types.  Can include ANY, */
-				/*%< defaults to all but SIG,SOA,NS if NULL */
+	bool grant;		/*%< is this a grant or a deny? */
+	dns_ssumatchtype_t matchtype;	/*%< which type of pattern match? */
+	dns_name_t *identity;		/*%< the identity to match */
+	dns_name_t *name;		/*%< the name being updated */
+	unsigned int ntypes;		/*%< number of data types covered */
+	dns_rdatatype_t *types;		/*%< the data types.  Can include */
+					/*   ANY. if NULL, defaults to all */
+					/*   types except SIG, SOA, and NS */
 	ISC_LINK(dns_ssurule_t) link;
 };
 
@@ -128,7 +131,7 @@ dns_ssutable_attach(dns_ssutable_t *source, dns_ssutable_t **targetp) {
 void
 dns_ssutable_detach(dns_ssutable_t **tablep) {
 	dns_ssutable_t *table;
-	isc_boolean_t done = ISC_FALSE;
+	bool done = false;
 
 	REQUIRE(tablep != NULL);
 	table = *tablep;
@@ -138,7 +141,7 @@ dns_ssutable_detach(dns_ssutable_t **tablep) {
 
 	INSIST(table->references > 0);
 	if (--table->references == 0)
-		done = ISC_TRUE;
+		done = true;
 	UNLOCK(&table->lock);
 
 	*tablep = NULL;
@@ -148,9 +151,9 @@ dns_ssutable_detach(dns_ssutable_t **tablep) {
 }
 
 isc_result_t
-dns_ssutable_addrule(dns_ssutable_t *table, isc_boolean_t grant,
-		     dns_name_t *identity, unsigned int matchtype,
-		     dns_name_t *name, unsigned int ntypes,
+dns_ssutable_addrule(dns_ssutable_t *table, bool grant,
+		     const dns_name_t *identity, dns_ssumatchtype_t matchtype,
+		     const dns_name_t *name, unsigned int ntypes,
 		     dns_rdatatype_t *types)
 {
 	dns_ssurule_t *rule;
@@ -160,8 +163,8 @@ dns_ssutable_addrule(dns_ssutable_t *table, isc_boolean_t grant,
 	REQUIRE(VALID_SSUTABLE(table));
 	REQUIRE(dns_name_isabsolute(identity));
 	REQUIRE(dns_name_isabsolute(name));
-	REQUIRE(matchtype <= DNS_SSUMATCHTYPE_MAX);
-	if (matchtype == DNS_SSUMATCHTYPE_WILDCARD)
+	REQUIRE(matchtype <= dns_ssumatchtype_max);
+	if (matchtype == dns_ssumatchtype_wildcard)
 		REQUIRE(dns_name_iswildcard(name));
 	if (ntypes > 0)
 		REQUIRE(types != NULL);
@@ -235,18 +238,18 @@ dns_ssutable_addrule(dns_ssutable_t *table, isc_boolean_t grant,
 	return (result);
 }
 
-static inline isc_boolean_t
+static inline bool
 isusertype(dns_rdatatype_t type) {
-	return (ISC_TF(type != dns_rdatatype_ns &&
-		       type != dns_rdatatype_soa &&
-		       type != dns_rdatatype_rrsig));
+	return (type != dns_rdatatype_ns &&
+		type != dns_rdatatype_soa &&
+		type != dns_rdatatype_rrsig);
 }
 
 static void
-reverse_from_address(dns_name_t *tcpself, isc_netaddr_t *tcpaddr) {
+reverse_from_address(dns_name_t *tcpself, const isc_netaddr_t *tcpaddr) {
 	char buf[16 * 4 + sizeof("IP6.ARPA.")];
 	isc_result_t result;
-	unsigned char *ap;
+	const unsigned char *ap;
 	isc_buffer_t b;
 	unsigned long l;
 
@@ -295,10 +298,10 @@ reverse_from_address(dns_name_t *tcpself, isc_netaddr_t *tcpaddr) {
 }
 
 static void
-stf_from_address(dns_name_t *stfself, isc_netaddr_t *tcpaddr) {
+stf_from_address(dns_name_t *stfself, const isc_netaddr_t *tcpaddr) {
 	char buf[sizeof("X.X.X.X.Y.Y.Y.Y.2.0.0.2.IP6.ARPA.")];
 	isc_result_t result;
-	unsigned char *ap;
+	const unsigned char *ap;
 	isc_buffer_t b;
 	unsigned long l;
 
@@ -336,21 +339,20 @@ stf_from_address(dns_name_t *stfself, isc_netaddr_t *tcpaddr) {
 	RUNTIME_CHECK(result == ISC_R_SUCCESS);
 }
 
-isc_boolean_t
-dns_ssutable_checkrules(dns_ssutable_t *table, dns_name_t *signer,
-			dns_name_t *name, isc_netaddr_t *addr,
+bool
+dns_ssutable_checkrules(dns_ssutable_t *table, const dns_name_t *signer,
+			const dns_name_t *name, const isc_netaddr_t *tcpaddr,
 			dns_rdatatype_t type, const dst_key_t *key)
 {
 	return (dns_ssutable_checkrules2
-		(table, signer, name, addr,
-		 addr == NULL ? ISC_FALSE : ISC_TRUE,
+		(table, signer, name, tcpaddr,
+		 tcpaddr == NULL ? false : true,
 		 NULL, type, key));
 }
-
-isc_boolean_t
-dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
-			 dns_name_t *name, isc_netaddr_t *addr,
-			 isc_boolean_t tcp, const dns_aclenv_t *env,
+bool
+dns_ssutable_checkrules2(dns_ssutable_t *table, const dns_name_t *signer,
+			 const dns_name_t *name, const isc_netaddr_t *addr,
+			 bool tcp, const dns_aclenv_t *env,
 			 dns_rdatatype_t type, const dst_key_t *key)
 {
 	dns_ssurule_t *rule;
@@ -368,20 +370,20 @@ dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
 	REQUIRE(addr == NULL || env != NULL);
 
 	if (signer == NULL && addr == NULL)
-		return (ISC_FALSE);
+		return (false);
 
 	for (rule = ISC_LIST_HEAD(table->rules);
 	     rule != NULL;
 	     rule = ISC_LIST_NEXT(rule, link))
 	{
 		switch (rule->matchtype) {
-		case DNS_SSUMATCHTYPE_NAME:
-		case DNS_SSUMATCHTYPE_LOCAL:
-		case DNS_SSUMATCHTYPE_SUBDOMAIN:
-		case DNS_SSUMATCHTYPE_WILDCARD:
-		case DNS_SSUMATCHTYPE_SELF:
-		case DNS_SSUMATCHTYPE_SELFSUB:
-		case DNS_SSUMATCHTYPE_SELFWILD:
+		case dns_ssumatchtype_name:
+		case dns_ssumatchtype_local:
+		case dns_ssumatchtype_subdomain:
+		case dns_ssumatchtype_wildcard:
+		case dns_ssumatchtype_self:
+		case dns_ssumatchtype_selfsub:
+		case dns_ssumatchtype_selfwild:
 			if (signer == NULL)
 				continue;
 			if (dns_name_iswildcard(rule->identity)) {
@@ -393,30 +395,35 @@ dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
 					continue;
 			}
 			break;
-		case DNS_SSUMATCHTYPE_SELFKRB5:
-		case DNS_SSUMATCHTYPE_SELFMS:
-		case DNS_SSUMATCHTYPE_SUBDOMAINKRB5:
-		case DNS_SSUMATCHTYPE_SUBDOMAINMS:
+		case dns_ssumatchtype_selfkrb5:
+		case dns_ssumatchtype_selfms:
+		case dns_ssumatchtype_selfsubkrb5:
+		case dns_ssumatchtype_selfsubms:
+		case dns_ssumatchtype_subdomainkrb5:
+		case dns_ssumatchtype_subdomainms:
 			if (signer == NULL)
 				continue;
 			break;
-		case DNS_SSUMATCHTYPE_TCPSELF:
-		case DNS_SSUMATCHTYPE_6TO4SELF:
+		case dns_ssumatchtype_tcpself:
+		case dns_ssumatchtype_6to4self:
 			if (!tcp || addr == NULL)
 				continue;
 			break;
+		case dns_ssumatchtype_external:
+		case dns_ssumatchtype_dlz:
+			break;
 		}
 
 		switch (rule->matchtype) {
-		case DNS_SSUMATCHTYPE_NAME:
+		case dns_ssumatchtype_name:
 			if (!dns_name_equal(name, rule->name))
 				continue;
 			break;
-		case DNS_SSUMATCHTYPE_SUBDOMAIN:
+		case dns_ssumatchtype_subdomain:
 			if (!dns_name_issubdomain(name, rule->name))
 				continue;
 			break;
-		case DNS_SSUMATCHTYPE_LOCAL:
+		case dns_ssumatchtype_local:
 			if (addr == NULL) {
 				continue;
 			}
@@ -439,19 +446,19 @@ dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
 				continue;
 			}
 			break;
-		case DNS_SSUMATCHTYPE_WILDCARD:
+		case dns_ssumatchtype_wildcard:
 			if (!dns_name_matcheswildcard(name, rule->name))
 				continue;
 			break;
-		case DNS_SSUMATCHTYPE_SELF:
+		case dns_ssumatchtype_self:
 			if (!dns_name_equal(signer, name))
 				continue;
 			break;
-		case DNS_SSUMATCHTYPE_SELFSUB:
+		case dns_ssumatchtype_selfsub:
 			if (!dns_name_issubdomain(name, signer))
 				continue;
 			break;
-		case DNS_SSUMATCHTYPE_SELFWILD:
+		case dns_ssumatchtype_selfwild:
 			wildcard = dns_fixedname_initname(&fixed);
 			result = dns_name_concatenate(dns_wildcardname, signer,
 						      wildcard, NULL);
@@ -460,31 +467,57 @@ dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
 			if (!dns_name_matcheswildcard(name, wildcard))
 				continue;
 			break;
-		case DNS_SSUMATCHTYPE_SELFKRB5:
-			if (!dst_gssapi_identitymatchesrealmkrb5(signer, name,
-							       rule->identity))
-				continue;
-			break;
-		case DNS_SSUMATCHTYPE_SELFMS:
-			if (!dst_gssapi_identitymatchesrealmms(signer, name,
-							       rule->identity))
-				continue;
-			break;
-		case DNS_SSUMATCHTYPE_SUBDOMAINKRB5:
+		case dns_ssumatchtype_selfkrb5:
+			if (dst_gssapi_identitymatchesrealmkrb5(signer, name,
+								rule->identity,
+								false))
+			{
+				break;
+			}
+			continue;
+		case dns_ssumatchtype_selfms:
+			if (dst_gssapi_identitymatchesrealmms(signer, name,
+							      rule->identity,
+							      false))
+			{
+				break;
+			}
+			continue;
+		case dns_ssumatchtype_selfsubkrb5:
+			if (dst_gssapi_identitymatchesrealmkrb5(signer, name,
+								rule->identity,
+								true))
+			{
+				break;
+			}
+			continue;
+		case dns_ssumatchtype_selfsubms:
+			if (dst_gssapi_identitymatchesrealmms(signer, name,
+							      rule->identity,
+							      true))
+				break;
+			continue;
+		case dns_ssumatchtype_subdomainkrb5:
 			if (!dns_name_issubdomain(name, rule->name))
 				continue;
-			if (!dst_gssapi_identitymatchesrealmkrb5(signer, NULL,
-							       rule->identity))
-				continue;
-			break;
-		case DNS_SSUMATCHTYPE_SUBDOMAINMS:
+			if (dst_gssapi_identitymatchesrealmkrb5(signer, NULL,
+								rule->identity,
+								false))
+			{
+				break;
+			}
+			continue;
+		case dns_ssumatchtype_subdomainms:
 			if (!dns_name_issubdomain(name, rule->name))
 				continue;
-			if (!dst_gssapi_identitymatchesrealmms(signer, NULL,
-							       rule->identity))
-				continue;
-			break;
-		case DNS_SSUMATCHTYPE_TCPSELF:
+			if (dst_gssapi_identitymatchesrealmms(signer, NULL,
+							      rule->identity,
+							      false))
+			{
+				break;
+			}
+			continue;
+		case dns_ssumatchtype_tcpself:
 			tcpself = dns_fixedname_initname(&fixed);
 			reverse_from_address(tcpself, addr);
 			if (dns_name_iswildcard(rule->identity)) {
@@ -498,7 +531,7 @@ dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
 			if (!dns_name_equal(tcpself, name))
 				continue;
 			break;
-		case DNS_SSUMATCHTYPE_6TO4SELF:
+		case dns_ssumatchtype_6to4self:
 			stfself = dns_fixedname_initname(&fixed);
 			stf_from_address(stfself, addr);
 			if (dns_name_iswildcard(rule->identity)) {
@@ -512,13 +545,13 @@ dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
 			if (!dns_name_equal(stfself, name))
 				continue;
 			break;
-		case DNS_SSUMATCHTYPE_EXTERNAL:
+		case dns_ssumatchtype_external:
 			if (!dns_ssu_external_match(rule->identity, signer,
 						    name, addr, type, key,
 						    table->mctx))
 				continue;
 			break;
-		case DNS_SSUMATCHTYPE_DLZ:
+		case dns_ssumatchtype_dlz:
 			if (!dns_dlz_ssumatch(table->dlzdatabase, signer,
 					      name, addr, type, key))
 				continue;
@@ -531,7 +564,7 @@ dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
 			 * checks will have already checked
 			 * the type.
 			 */
-			if (rule->matchtype != DNS_SSUMATCHTYPE_DLZ &&
+			if (rule->matchtype != dns_ssumatchtype_dlz &&
 			    !isusertype(type))
 				continue;
 		} else {
@@ -546,10 +579,10 @@ dns_ssutable_checkrules2(dns_ssutable_t *table, dns_name_t *signer,
 		return (rule->grant);
 	}
 
-	return (ISC_FALSE);
+	return (false);
 }
 
-isc_boolean_t
+bool
 dns_ssurule_isgrant(const dns_ssurule_t *rule) {
 	REQUIRE(VALID_SSURULE(rule));
 	return (rule->grant);
@@ -625,8 +658,8 @@ dns_ssutable_createdlz(isc_mem_t *mctx, dns_ssutable_t **tablep,
 	rule->identity = NULL;
 	rule->name = NULL;
 	rule->types = NULL;
-	rule->grant = ISC_TRUE;
-	rule->matchtype = DNS_SSUMATCHTYPE_DLZ;
+	rule->grant = true;
+	rule->matchtype = dns_ssumatchtype_dlz;
 	rule->ntypes = 0;
 	rule->types = NULL;
 	rule->magic = SSURULEMAGIC;
@@ -656,8 +689,12 @@ dns_ssu_mtypefromstring(const char *str, dns_ssumatchtype_t *mtype) {
 		*mtype = dns_ssumatchtype_selfwild;
 	} else if (strcasecmp(str, "ms-self") == 0) {
 		*mtype = dns_ssumatchtype_selfms;
+	} else if (strcasecmp(str, "ms-selfsub") == 0) {
+		*mtype = dns_ssumatchtype_selfsubms;
 	} else if (strcasecmp(str, "krb5-self") == 0) {
 		*mtype = dns_ssumatchtype_selfkrb5;
+	} else if (strcasecmp(str, "krb5-selfsub") == 0) {
+		*mtype = dns_ssumatchtype_selfsubkrb5;
 	} else if (strcasecmp(str, "ms-subdomain") == 0) {
 		*mtype = dns_ssumatchtype_subdomainms;
 	} else if (strcasecmp(str, "krb5-subdomain") == 0) {
