@@ -22,6 +22,7 @@
 
 #include <errno.h>
 #include <fcntl.h>
+#include <stdbool.h>
 #include <stdio.h>
 
 #include <isc/buffer.h>
@@ -88,11 +89,11 @@
 
 /*% Entropy Pool */
 typedef struct {
-	isc_uint32_t	cursor;		/*%< current add point in the pool */
-	isc_uint32_t	entropy;	/*%< current entropy estimate in bits */
-	isc_uint32_t	pseudo;		/*%< bits extracted in pseudorandom */
-	isc_uint32_t	rotate;		/*%< how many bits to rotate by */
-	isc_uint32_t	pool[RND_POOLWORDS];	/*%< random pool data */
+	uint32_t	cursor;		/*%< current add point in the pool */
+	uint32_t	entropy;	/*%< current entropy estimate in bits */
+	uint32_t	pseudo;		/*%< bits extracted in pseudorandom */
+	uint32_t	rotate;		/*%< how many bits to rotate by */
+	uint32_t	pool[RND_POOLWORDS];	/*%< random pool data */
 } isc_entropypool_t;
 
 struct isc_entropy {
@@ -100,10 +101,10 @@ struct isc_entropy {
 	isc_mem_t		       *mctx;
 	isc_mutex_t			lock;
 	unsigned int			refcnt;
-	isc_uint32_t			initialized;
-	isc_uint32_t			initcount;
+	uint32_t			initialized;
+	uint32_t			initcount;
 	isc_entropypool_t		pool;
-	isc_boolean_t			usehook;
+	bool			usehook;
 	unsigned int			nsources;
 	isc_entropysource_t	       *nextsource;
 	ISC_LIST(isc_entropysource_t)	sources;
@@ -114,12 +115,12 @@ static isc_entropy_getdata_t hook;
 
 /*% Sample Queue */
 typedef struct {
-	isc_uint32_t	last_time;	/*%< last time recorded */
-	isc_uint32_t	last_delta;	/*%< last delta value */
-	isc_uint32_t	last_delta2;	/*%< last delta2 value */
-	isc_uint32_t	nsamples;	/*%< number of samples filled in */
-	isc_uint32_t   *samples;	/*%< the samples */
-	isc_uint32_t   *extra;		/*%< extra samples added in */
+	uint32_t	last_time;	/*%< last time recorded */
+	uint32_t	last_delta;	/*%< last delta value */
+	uint32_t	last_delta2;	/*%< last delta2 value */
+	uint32_t	nsamples;	/*%< number of samples filled in */
+	uint32_t   *samples;	/*%< the samples */
+	uint32_t   *extra;		/*%< extra samples added in */
 } sample_queue_t;
 
 typedef struct {
@@ -127,7 +128,7 @@ typedef struct {
 } isc_entropysamplesource_t;
 
 typedef struct {
-	isc_boolean_t		start_called;
+	bool		start_called;
 	isc_entropystart_t	startfunc;
 	isc_entropyget_t	getfunc;
 	isc_entropystop_t	stopfunc;
@@ -143,11 +144,11 @@ struct isc_entropysource {
 	unsigned int	magic;
 	unsigned int	type;
 	isc_entropy_t  *ent;
-	isc_uint32_t	total;		/*%< entropy from this source */
+	uint32_t	total;		/*%< entropy from this source */
 	ISC_LINK(isc_entropysource_t)	link;
 	char		name[32];
-	isc_boolean_t	bad;
-	isc_boolean_t	warn_keyboard;
+	bool	bad;
+	bool	warn_keyboard;
 	isc_keyboard_t	kbd;
 	union {
 		isc_entropysamplesource_t	sample;
@@ -179,7 +180,7 @@ struct isc_entropysource {
  * include this file.
  */
 static void
-fillpool(isc_entropy_t *, unsigned int, isc_boolean_t);
+fillpool(isc_entropy_t *, unsigned int, bool);
 
 static int
 wait_for_sources(isc_entropy_t *);
@@ -226,7 +227,7 @@ samplesource_allocate(isc_entropy_t *ent, sample_queue_t *sq) {
  * very large.
  */
 static inline void
-add_entropy(isc_entropy_t *ent, isc_uint32_t entropy) {
+add_entropy(isc_entropy_t *ent, uint32_t entropy) {
 	/* clamp input.  Yes, this must be done. */
 	entropy = ISC_MIN(entropy, RND_POOLBITS);
 	/* Add in the entropy we already have. */
@@ -239,7 +240,7 @@ add_entropy(isc_entropy_t *ent, isc_uint32_t entropy) {
  * Decrement the amount of entropy the pool has.
  */
 static inline void
-subtract_entropy(isc_entropy_t *ent, isc_uint32_t entropy) {
+subtract_entropy(isc_entropy_t *ent, uint32_t entropy) {
 	entropy = ISC_MIN(entropy, ent->pool.entropy);
 	ent->pool.entropy -= entropy;
 }
@@ -249,7 +250,7 @@ subtract_entropy(isc_entropy_t *ent, isc_uint32_t entropy) {
  * very large.
  */
 static inline void
-add_pseudo(isc_entropy_t *ent, isc_uint32_t pseudo) {
+add_pseudo(isc_entropy_t *ent, uint32_t pseudo) {
 	/* clamp input.  Yes, this must be done. */
 	pseudo = ISC_MIN(pseudo, RND_POOLBITS * 8);
 	/* Add in the pseudo we already have. */
@@ -262,7 +263,7 @@ add_pseudo(isc_entropy_t *ent, isc_uint32_t pseudo) {
  * Decrement the amount of pseudo the pool has.
  */
 static inline void
-subtract_pseudo(isc_entropy_t *ent, isc_uint32_t pseudo) {
+subtract_pseudo(isc_entropy_t *ent, uint32_t pseudo) {
 	pseudo = ISC_MIN(pseudo, ent->pool.pseudo);
 	ent->pool.pseudo -= pseudo;
 }
@@ -271,7 +272,7 @@ subtract_pseudo(isc_entropy_t *ent, isc_uint32_t pseudo) {
  * Add one word to the pool, rotating the input as needed.
  */
 static inline void
-entropypool_add_word(isc_entropypool_t *rp, isc_uint32_t val) {
+entropypool_add_word(isc_entropypool_t *rp, uint32_t val) {
 	/*
 	 * Steal some values out of the pool, and xor them into the
 	 * word we were given.
@@ -312,11 +313,11 @@ entropypool_add_word(isc_entropypool_t *rp, isc_uint32_t val) {
  */
 static void
 entropypool_adddata(isc_entropy_t *ent, void *p, unsigned int len,
-		    isc_uint32_t entropy)
+		    uint32_t entropy)
 {
-	isc_uint32_t val;
+	uint32_t val;
 	unsigned long addr;
-	isc_uint8_t *buf;
+	uint8_t *buf;
 
 	/* Silly MSVC in 64 bit mode complains here... */
 #ifdef _WIN64
@@ -346,7 +347,7 @@ entropypool_adddata(isc_entropy_t *ent, void *p, unsigned int len,
 	}
 
 	for (; len > 3; len -= 4) {
-		val = *((isc_uint32_t *)buf);
+		val = *((uint32_t *)buf);
 
 		entropypool_add_word(&ent->pool, val);
 		buf += 4;
@@ -399,10 +400,10 @@ reseed(isc_entropy_t *ent) {
 }
 
 static inline unsigned int
-estimate_entropy(sample_queue_t *sq, isc_uint32_t t) {
-	isc_int32_t		delta;
-	isc_int32_t		delta2;
-	isc_int32_t		delta3;
+estimate_entropy(sample_queue_t *sq, uint32_t t) {
+	int32_t		delta;
+	int32_t		delta2;
+	int32_t		delta3;
 
 	/*!
 	 * If the time counter has overflowed, calculate the real difference.
@@ -487,7 +488,7 @@ crunchsamples(isc_entropy_t *ent, sample_queue_t *sq) {
 
 static unsigned int
 get_from_callback(isc_entropysource_t *source, unsigned int desired,
-		  isc_boolean_t blocking)
+		  bool blocking)
 {
 	isc_entropy_t *ent = source->ent;
 	isc_cbsource_t *cbs = &source->sources.callback;
@@ -505,7 +506,7 @@ get_from_callback(isc_entropysource_t *source, unsigned int desired,
 		result = cbs->startfunc(source, cbs->arg, blocking);
 		if (result != ISC_R_SUCCESS)
 			return (0);
-		cbs->start_called = ISC_TRUE;
+		cbs->start_called = true;
 	}
 
 	added = 0;
@@ -519,7 +520,7 @@ get_from_callback(isc_entropysource_t *source, unsigned int desired,
 			result = ISC_R_SUCCESS;
 		} else if (result != ISC_R_SUCCESS &&
 			   result != ISC_R_NOTBLOCKING)
-			source->bad = ISC_TRUE;
+			source->bad = true;
 
 	}
 
@@ -544,17 +545,17 @@ isc_entropy_getdata(isc_entropy_t *ent, void *data, unsigned int length,
 	unsigned int i;
 	isc_sha1_t hash;
 	unsigned char digest[ISC_SHA1_DIGESTLENGTH];
-	isc_uint32_t remain, deltae, count, total;
-	isc_uint8_t *buf;
-	isc_boolean_t goodonly, partial, blocking;
+	uint32_t remain, deltae, count, total;
+	uint8_t *buf;
+	bool goodonly, partial, blocking;
 
 	REQUIRE(VALID_ENTROPY(ent));
 	REQUIRE(data != NULL);
 	REQUIRE(length > 0);
 
-	goodonly = ISC_TF((flags & ISC_ENTROPY_GOODONLY) != 0);
-	partial = ISC_TF((flags & ISC_ENTROPY_PARTIAL) != 0);
-	blocking = ISC_TF((flags & ISC_ENTROPY_BLOCKING) != 0);
+	goodonly = (flags & ISC_ENTROPY_GOODONLY);
+	partial = (flags & ISC_ENTROPY_PARTIAL);
+	blocking = (flags & ISC_ENTROPY_BLOCKING);
 
 	REQUIRE(!partial || returned != NULL);
 
@@ -590,7 +591,7 @@ isc_entropy_getdata(isc_entropy_t *ent, void *data, unsigned int length,
 			 * pool full.
 			 */
 			if (ent->pool.entropy >= THRESHOLD_BITS)
-				fillpool(ent, fillcount, ISC_FALSE);
+				fillpool(ent, fillcount, false);
 			else
 				fillpool(ent, fillcount, blocking);
 
@@ -612,7 +613,7 @@ isc_entropy_getdata(isc_entropy_t *ent, void *data, unsigned int length,
 			if (ent->initialized < THRESHOLD_BITS)
 				fillpool(ent, THRESHOLD_BITS, blocking);
 			else
-				fillpool(ent, 0, ISC_FALSE);
+				fillpool(ent, 0, false);
 
 			/*
 			 * If we've not initialized with enough good random
@@ -716,7 +717,7 @@ isc_entropy_create(isc_mem_t *mctx, isc_entropy_t **entp) {
 	ent->refcnt = 1;
 	ent->initialized = 0;
 	ent->initcount = 0;
-	ent->usehook = ISC_FALSE;
+	ent->usehook = false;
 	ent->magic = ENTROPY_MAGIC;
 
 	isc_entropypool_init(&ent->pool);
@@ -764,7 +765,7 @@ destroysource(isc_entropysource_t **sourcep) {
 		cbs = &source->sources.callback;
 		if (cbs->start_called && cbs->stopfunc != NULL) {
 			cbs->stopfunc(source, cbs->arg);
-			cbs->start_called = ISC_FALSE;
+			cbs->start_called = false;
 		}
 		samplequeue_release(ent, &cbs->samplequeue);
 		break;
@@ -774,12 +775,12 @@ destroysource(isc_entropysource_t **sourcep) {
 	isc_mem_put(ent->mctx, source, sizeof(*source));
 }
 
-static inline isc_boolean_t
+static inline bool
 destroy_check(isc_entropy_t *ent) {
 	isc_entropysource_t *source;
 
 	if (ent->refcnt > 0)
-		return (ISC_FALSE);
+		return (false);
 
 	source = ISC_LIST_HEAD(ent->sources);
 	while (source != NULL) {
@@ -788,12 +789,12 @@ destroy_check(isc_entropy_t *ent) {
 		case ENTROPY_SOURCETYPE_USOCKET:
 			break;
 		default:
-			return (ISC_FALSE);
+			return (false);
 		}
 		source = ISC_LIST_NEXT(source, link);
 	}
 
-	return (ISC_TRUE);
+	return (true);
 }
 
 static void
@@ -846,7 +847,7 @@ void
 isc_entropy_destroysource(isc_entropysource_t **sourcep) {
 	isc_entropysource_t *source;
 	isc_entropy_t *ent;
-	isc_boolean_t killit;
+	bool killit;
 
 	REQUIRE(sourcep != NULL);
 	REQUIRE(VALID_SOURCE(*sourcep));
@@ -892,7 +893,7 @@ isc_entropy_createcallbacksource(isc_entropy_t *ent,
 		result = ISC_R_NOMEMORY;
 		goto errout;
 	}
-	source->bad = ISC_FALSE;
+	source->bad = false;
 
 	cbs = &source->sources.callback;
 
@@ -900,7 +901,7 @@ isc_entropy_createcallbacksource(isc_entropy_t *ent,
 	if (result != ISC_R_SUCCESS)
 		goto errout;
 
-	cbs->start_called = ISC_FALSE;
+	cbs->start_called = false;
 	cbs->startfunc = start;
 	cbs->getfunc = get;
 	cbs->stopfunc = stop;
@@ -951,7 +952,7 @@ isc_entropy_stopcallbacksources(isc_entropy_t *ent) {
 			cbs = &source->sources.callback;
 			if (cbs->start_called && cbs->stopfunc != NULL) {
 				cbs->stopfunc(source, cbs->arg);
-				cbs->start_called = ISC_FALSE;
+				cbs->start_called = false;
 			}
 		}
 
@@ -1021,7 +1022,7 @@ isc_entropy_createsamplesource(isc_entropy_t *ent,
  * queue was full when this function was called.
  */
 static isc_result_t
-addsample(sample_queue_t *sq, isc_uint32_t sample, isc_uint32_t extra) {
+addsample(sample_queue_t *sq, uint32_t sample, uint32_t extra) {
 	if (sq->nsamples >= RND_EVENTQSIZE)
 		return (ISC_R_NOMORE);
 
@@ -1036,8 +1037,8 @@ addsample(sample_queue_t *sq, isc_uint32_t sample, isc_uint32_t extra) {
 }
 
 isc_result_t
-isc_entropy_addsample(isc_entropysource_t *source, isc_uint32_t sample,
-		      isc_uint32_t extra)
+isc_entropy_addsample(isc_entropysource_t *source, uint32_t sample,
+		      uint32_t extra)
 {
 	isc_entropy_t *ent;
 	sample_queue_t *sq;
@@ -1063,8 +1064,8 @@ isc_entropy_addsample(isc_entropysource_t *source, isc_uint32_t sample,
 }
 
 isc_result_t
-isc_entropy_addcallbacksample(isc_entropysource_t *source, isc_uint32_t sample,
-			      isc_uint32_t extra)
+isc_entropy_addcallbacksample(isc_entropysource_t *source, uint32_t sample,
+			      uint32_t extra)
 {
 	sample_queue_t *sq;
 	isc_result_t result;
@@ -1080,7 +1081,7 @@ isc_entropy_addcallbacksample(isc_entropysource_t *source, isc_uint32_t sample,
 
 void
 isc_entropy_putdata(isc_entropy_t *ent, void *data, unsigned int length,
-		    isc_uint32_t entropy)
+		    uint32_t entropy)
 {
 	REQUIRE(VALID_ENTROPY(ent));
 
@@ -1148,7 +1149,7 @@ isc_entropy_attach(isc_entropy_t *ent, isc_entropy_t **entp) {
 void
 isc_entropy_detach(isc_entropy_t **entp) {
 	isc_entropy_t *ent;
-	isc_boolean_t killit;
+	bool killit;
 
 	REQUIRE(entp != NULL && VALID_ENTROPY(*entp));
 	ent = *entp;
@@ -1168,13 +1169,13 @@ isc_entropy_detach(isc_entropy_t **entp) {
 }
 
 static isc_result_t
-kbdstart(isc_entropysource_t *source, void *arg, isc_boolean_t blocking) {
+kbdstart(isc_entropysource_t *source, void *arg, bool blocking) {
 	/*
 	 * The intent of "first" is to provide a warning message only once
 	 * during the run of a program that might try to gather keyboard
 	 * entropy multiple times.
 	 */
-	static isc_boolean_t first = ISC_TRUE;
+	static bool first = true;
 
 	UNUSED(arg);
 
@@ -1186,7 +1187,7 @@ kbdstart(isc_entropysource_t *source, void *arg, isc_boolean_t blocking) {
 			fprintf(stderr, "You must use the keyboard to create "
 				"entropy, since your system is lacking\n"
 				"/dev/random (or equivalent)\n\n");
-		first = ISC_FALSE;
+		first = false;
 	}
 	fprintf(stderr, "start typing:\n");
 
@@ -1205,11 +1206,11 @@ kbdstop(isc_entropysource_t *source, void *arg) {
 }
 
 static isc_result_t
-kbdget(isc_entropysource_t *source, void *arg, isc_boolean_t blocking) {
+kbdget(isc_entropysource_t *source, void *arg, bool blocking) {
 	isc_result_t result;
 	isc_time_t t;
-	isc_uint32_t sample;
-	isc_uint32_t extra;
+	uint32_t sample;
+	uint32_t extra;
 	unsigned char c;
 
 	UNUSED(arg);
@@ -1244,7 +1245,7 @@ isc_entropy_usebestsource(isc_entropy_t *ectx, isc_entropysource_t **source,
 {
 	isc_result_t result;
 	isc_result_t final_result = ISC_R_NOENTROPY;
-	isc_boolean_t userfile = ISC_TRUE;
+	bool userfile = true;
 
 	REQUIRE(VALID_ENTROPY(ectx));
 	REQUIRE(source != NULL && *source == NULL);
@@ -1260,7 +1261,7 @@ isc_entropy_usebestsource(isc_entropy_t *ectx, isc_entropysource_t **source,
 #ifdef PATH_RANDOMDEV
 	if (randomfile == NULL) {
 		randomfile = PATH_RANDOMDEV;
-		userfile = ISC_FALSE;
+		userfile = false;
 	}
 #endif
 
@@ -1281,8 +1282,7 @@ isc_entropy_usebestsource(isc_entropy_t *ectx, isc_entropysource_t **source,
 							  NULL, source);
 		if (result == ISC_R_SUCCESS)
 			(*source)->warn_keyboard =
-				ISC_TF(use_keyboard ==
-				       ISC_ENTROPY_KEYBOARDMAYBE);
+				(use_keyboard == ISC_ENTROPY_KEYBOARDMAYBE);
 
 		if (final_result != ISC_R_SUCCESS)
 			final_result = result;
@@ -1298,7 +1298,7 @@ isc_entropy_usebestsource(isc_entropy_t *ectx, isc_entropysource_t **source,
 }
 
 void
-isc_entropy_usehook(isc_entropy_t *ectx, isc_boolean_t onoff) {
+isc_entropy_usehook(isc_entropy_t *ectx, bool onoff) {
 	REQUIRE(VALID_ENTROPY(ectx));
 
 	LOCK(&ectx->lock);
