@@ -1,25 +1,23 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * See the COPYRIGHT file distributed with this work for additional
+ * information regarding copyright ownership.
  */
 
-/* $Id$ */
 
 /*! \file */
 
 #include <config.h>
 
+#include <inttypes.h>
+#include <stdbool.h>
+#include <string.h>
+
 #include <isc/buffer.h>
 #include <isc/httpd.h>
 #include <isc/mem.h>
@@ -30,7 +28,9 @@
 #include <isc/time.h>
 #include <isc/util.h>
 
-#include <string.h>
+#ifdef HAVE_ZLIB
+#include <zlib.h>
+#endif
 
 /*%
  * TODO:
@@ -63,6 +63,7 @@
 #define HTTPD_CLOSE		0x0001 /* Got a Connection: close header */
 #define HTTPD_FOUNDHOST		0x0002 /* Got a Host: header */
 #define HTTPD_KEEPALIVE		0x0004 /* Got a Connection: Keep-Alive */
+#define HTTPD_ACCEPT_DEFLATE   0x0008
 
 /*% http client */
 struct isc_httpd {
@@ -75,7 +76,7 @@ struct isc_httpd {
 	 * Received data state.
 	 */
 	char			recvbuf[HTTP_RECVLEN]; /*%< receive buffer */
-	isc_uint32_t		recvlen;	/*%< length recv'd */
+	uint32_t		recvlen;	/*%< length recv'd */
 	char		       *headers;	/*%< set in process_request() */
 	unsigned int		method;
 	char		       *url;
@@ -97,23 +98,25 @@ struct isc_httpd {
 	 * to the client.
 	 *
 	 * The bufflist is the list of buffers we are currently transmitting.
-	 * The headerdata is where we render our headers to.  If we run out of
+	 * The headerbuffer is where we render our headers to.  If we run out of
 	 * space when rendering a header, we will change the size of our
 	 * buffer.  We will not free it until we are finished, and will
 	 * allocate an additional HTTP_SENDGROW bytes per header space grow.
 	 *
-	 * We currently use two buffers total, one for the headers (which
-	 * we manage) and another for the client to fill in (which it manages,
+	 * We currently use three buffers total, one for the headers (which
+	 * we manage), another for the client to fill in (which it manages,
 	 * it provides the space for it, etc) -- we will pass that buffer
 	 * structure back to the caller, who is responsible for managing the
 	 * space it may have allocated as backing store for it.  This second
 	 * buffer is bodybuffer, and we only allocate the buffer itself, not
 	 * the backing store.
+	 * The third buffer is compbuffer, managed by us, that contains the
+	 * compressed HTTP data, if compression is used.
+	 *
 	 */
 	isc_bufferlist_t	bufflist;
-	char		       *headerdata; /*%< send header buf */
-	unsigned int		headerlen;  /*%< current header buffer size */
 	isc_buffer_t		headerbuffer;
+	isc_buffer_t		compbuffer;
 
 	const char	       *mimetype;
 	unsigned int		retcode;
@@ -211,10 +214,13 @@ static void reset_client(isc_httpd_t *httpd);
 static isc_httpdaction_t render_404;
 static isc_httpdaction_t render_500;
 
+static void (*finishhook)(void) = NULL;
+
 static void
 destroy_client(isc_httpd_t **httpdp) {
 	isc_httpd_t *httpd = *httpdp;
 	isc_httpdmgr_t *httpdmgr = httpd->mgr;
+	isc_region_t r;
 
 	*httpdp = NULL;
 
@@ -223,14 +229,23 @@ destroy_client(isc_httpd_t **httpdp) {
 	isc_socket_detach(&httpd->sock);
 	ISC_LIST_UNLINK(httpdmgr->running, httpd, link);
 
-	if (httpd->headerlen > 0)
-		isc_mem_put(httpdmgr->mctx, httpd->headerdata,
-			    httpd->headerlen);
+	isc_buffer_region(&httpd->headerbuffer, &r);
+	if (r.length > 0) {
+		isc_mem_put(httpdmgr->mctx, r.base, r.length);
+	}
+
+	isc_buffer_region(&httpd->compbuffer, &r);
+	if (r.length > 0) {
+		isc_mem_put(httpdmgr->mctx, r.base, r.length);
+	}
 
 	isc_mem_put(httpdmgr->mctx, httpd, sizeof(isc_httpd_t));
 
 	UNLOCK(&httpdmgr->lock);
 
+	if (finishhook != NULL)
+		finishhook();
+
 	httpdmgr_destroy(httpdmgr);
 }
 
@@ -364,7 +379,7 @@ httpdmgr_destroy(isc_httpdmgr_t *httpdmgr) {
  * Look for the given header in headers.
  * If value is specified look for it terminated with a character in eov.
  */
-static isc_boolean_t
+static bool
 have_header(isc_httpd_t *httpd, const char *header, const char *value,
 	    const char *eov)
 {
@@ -393,13 +408,13 @@ have_header(isc_httpd_t *httpd, const char *header, const char *value,
 			if (h == NULL || (nl != NULL && nl < h))
 				h = nl;
 			if (h == NULL)
-				return (ISC_FALSE);
+				return (false);
 			h++;
 			continue;
 		}
 
 		if (value == NULL)
-			return (ISC_TRUE);
+			return (true);
 
 		/*
 		 * Skip optional leading white space.
@@ -413,7 +428,7 @@ have_header(isc_httpd_t *httpd, const char *header, const char *value,
 		while (*h != 0 && *h != '\r' && *h != '\n') {
 			if (strncasecmp(h, value, vlen) == 0)
 				if (strchr(eov, h[vlen]) != NULL)
-					return (ISC_TRUE);
+					return (true);
 			/*
 			 * Skip to next token.
 			 */
@@ -423,7 +438,7 @@ have_header(isc_httpd_t *httpd, const char *header, const char *value,
 			if (h[0] != 0)
 				h++;
 		}
-		return (ISC_FALSE);
+		return (false);
 	}
 }
 
@@ -569,6 +584,14 @@ process_request(isc_httpd_t *httpd, int length) {
 			httpd->flags |= HTTPD_CLOSE;
 	}
 
+	/*
+	 * Check for Accept-Encoding:
+	 */
+#ifdef HAVE_ZLIB
+	if (have_header(httpd, "Accept-Encoding:", "deflate", ";, \t\r\n"))
+		httpd->flags |= HTTPD_ACCEPT_DEFLATE;
+#endif
+
 	/*
 	 * Standards compliance hooks here.
 	 */
@@ -589,6 +612,7 @@ isc_httpd_accept(isc_task_t *task, isc_event_t *ev) {
 	isc_region_t r;
 	isc_socket_newconnev_t *nev = (isc_socket_newconnev_t *)ev;
 	isc_sockaddr_t peeraddr;
+	char *headerdata;
 
 	ENTER("accept");
 
@@ -635,18 +659,17 @@ isc_httpd_accept(isc_task_t *task, isc_event_t *ev) {
 	/*
 	 * Initialize the buffer for our headers.
 	 */
-	httpd->headerdata = isc_mem_get(httpdmgr->mctx, HTTP_SENDGROW);
-	if (httpd->headerdata == NULL) {
+	headerdata = isc_mem_get(httpdmgr->mctx, HTTP_SENDGROW);
+	if (headerdata == NULL) {
 		isc_mem_put(httpdmgr->mctx, httpd, sizeof(isc_httpd_t));
 		isc_socket_detach(&nev->newsocket);
 		goto requeue;
 	}
-	httpd->headerlen = HTTP_SENDGROW;
-	isc_buffer_init(&httpd->headerbuffer, httpd->headerdata,
-			httpd->headerlen);
+	isc_buffer_init(&httpd->headerbuffer, headerdata, HTTP_SENDGROW);
 
 	ISC_LIST_INIT(httpd->bufflist);
 
+	isc_buffer_initnull(&httpd->compbuffer);
 	isc_buffer_initnull(&httpd->bodybuffer);
 	reset_client(httpd);
 
@@ -728,6 +751,93 @@ render_500(const char *url, isc_httpdurl_t *urlinfo,
 	return (ISC_R_SUCCESS);
 }
 
+#ifdef HAVE_ZLIB
+/*%<
+ * Reallocates compbuffer to size, does nothing if compbuffer is already
+ * larger than size.
+ *
+ * Requires:
+ *\li	httpd a valid isc_httpd_t object
+ *
+ * Returns:
+ *\li	#ISC_R_SUCCESS		-- all is well.
+ *\li	#ISC_R_NOMEMORY		-- not enough memory to extend buffer
+ */
+static isc_result_t
+alloc_compspace(isc_httpd_t *httpd, unsigned int size) {
+	char *newspace;
+	isc_region_t r;
+
+	isc_buffer_region(&httpd->compbuffer, &r);
+	if (size < r.length) {
+		return (ISC_R_SUCCESS);
+	}
+
+	newspace = isc_mem_get(httpd->mgr->mctx, size);
+	if (newspace == NULL)
+		return (ISC_R_NOMEMORY);
+	isc_buffer_reinit(&httpd->compbuffer, newspace, size);
+
+	if (r.base != NULL) {
+		isc_mem_put(httpd->mgr->mctx, r.base, r.length);
+	}
+
+	return (ISC_R_SUCCESS);
+}
+
+/*%<
+ * Tries to compress httpd->bodybuffer to httpd->compbuffer, extending it
+ * if necessary.
+ *
+ * Requires:
+ *\li	httpd a valid isc_httpd_t object
+ *
+ * Returns:
+ *\li	#ISC_R_SUCCESS	  -- all is well.
+ *\li	#ISC_R_NOMEMORY	  -- not enough memory to compress data
+ *\li	#ISC_R_FAILURE	  -- error during compression or compressed
+ *			     data would be larger than input data
+ */
+static isc_result_t
+isc_httpd_compress(isc_httpd_t *httpd) {
+	z_stream zstr;
+	isc_region_t r;
+	isc_result_t result;
+	int ret;
+	int inputlen;
+
+	inputlen = isc_buffer_usedlength(&httpd->bodybuffer);
+	result = alloc_compspace(httpd, inputlen);
+	if (result != ISC_R_SUCCESS) {
+		return result;
+	}
+	isc_buffer_region(&httpd->compbuffer, &r);
+
+	/*
+	 * We're setting output buffer size to input size so it fails if the
+	 * compressed data size would be bigger than the input size.
+	 */
+	memset(&zstr, 0, sizeof(zstr));
+	zstr.total_in = zstr.avail_in =
+			zstr.total_out = zstr.avail_out = inputlen;
+
+	zstr.next_in = isc_buffer_base(&httpd->bodybuffer);
+	zstr.next_out = r.base;
+
+	ret = deflateInit(&zstr, Z_DEFAULT_COMPRESSION);
+	if (ret == Z_OK) {
+		ret = deflate(&zstr, Z_FINISH);
+	}
+	deflateEnd(&zstr);
+	if (ret == Z_STREAM_END) {
+		isc_buffer_add(&httpd->compbuffer, inputlen - zstr.avail_out);
+		return (ISC_R_SUCCESS);
+	} else {
+		return (ISC_R_FAILURE);
+	}
+}
+#endif
+
 static void
 isc_httpd_recvdone(isc_task_t *task, isc_event_t *ev) {
 	isc_region_t r;
@@ -736,6 +846,7 @@ isc_httpd_recvdone(isc_task_t *task, isc_event_t *ev) {
 	isc_socketevent_t *sev = (isc_socketevent_t *)ev;
 	isc_httpdurl_t *url;
 	isc_time_t now;
+	bool is_compressed = false;
 	char datebuf[ISC_FORMATHTTPTIMESTAMP_SIZE];
 
 	ENTER("recv");
@@ -811,6 +922,15 @@ isc_httpd_recvdone(isc_task_t *task, isc_event_t *ev) {
 		RUNTIME_CHECK(result == ISC_R_SUCCESS);
 	}
 
+#ifdef HAVE_ZLIB
+	if (httpd->flags & HTTPD_ACCEPT_DEFLATE) {
+			result = isc_httpd_compress(httpd);
+			if (result == ISC_R_SUCCESS) {
+				is_compressed = true;
+			}
+	}
+#endif
+
 	isc_httpd_response(httpd);
 	if ((httpd->flags & HTTPD_KEEPALIVE) != 0)
 		isc_httpd_addheader(httpd, "Connection", "Keep-Alive");
@@ -831,8 +951,16 @@ isc_httpd_recvdone(isc_task_t *task, isc_event_t *ev) {
 	}
 
 	isc_httpd_addheader(httpd, "Server: libisc", NULL);
-	isc_httpd_addheaderuint(httpd, "Content-Length",
-				isc_buffer_usedlength(&httpd->bodybuffer));
+
+	if (is_compressed == true) {
+		isc_httpd_addheader(httpd, "Content-Encoding", "deflate");
+		isc_httpd_addheaderuint(httpd, "Content-Length",
+					isc_buffer_usedlength(&httpd->compbuffer));
+	} else {
+		isc_httpd_addheaderuint(httpd, "Content-Length",
+		isc_buffer_usedlength(&httpd->bodybuffer));
+	}
+
 	isc_httpd_endheaders(httpd);  /* done */
 
 	ISC_LIST_APPEND(httpd->bufflist, &httpd->headerbuffer, link);
@@ -841,8 +969,13 @@ isc_httpd_recvdone(isc_task_t *task, isc_event_t *ev) {
 	 * rendered into it.  If no data is present, we won't do anything
 	 * with the buffer.
 	 */
-	if (isc_buffer_length(&httpd->bodybuffer) > 0)
-		ISC_LIST_APPEND(httpd->bufflist, &httpd->bodybuffer, link);
+	if (is_compressed == true) {
+		ISC_LIST_APPEND(httpd->bufflist, &httpd->compbuffer, link);
+	} else {
+		if (isc_buffer_length(&httpd->bodybuffer) > 0) {
+				ISC_LIST_APPEND(httpd->bufflist, &httpd->bodybuffer, link);
+		}
+	}
 
 	/* check return code? */
 	(void)isc_socket_sendv(httpd->sock, &httpd->bufflist, task,
@@ -886,14 +1019,15 @@ grow_headerspace(isc_httpd_t *httpd) {
 	unsigned int newlen;
 	isc_region_t r;
 
-	newlen = httpd->headerlen + HTTP_SENDGROW;
+	isc_buffer_region(&httpd->headerbuffer, &r);
+	newlen = r.length + HTTP_SENDGROW;
 	if (newlen > HTTP_SEND_MAXLEN)
 		return (ISC_R_NOSPACE);
 
 	newspace = isc_mem_get(httpd->mgr->mctx, newlen);
 	if (newspace == NULL)
 		return (ISC_R_NOMEMORY);
-	isc_buffer_region(&httpd->headerbuffer, &r);
+
 	isc_buffer_reinit(&httpd->headerbuffer, newspace, newlen);
 
 	isc_mem_put(httpd->mgr->mctx, r.base, r.length);
@@ -916,13 +1050,9 @@ isc_httpd_response(isc_httpd_t *httpd) {
 			return (result);
 	}
 
-	snprintf(isc_buffer_used(&httpd->headerbuffer),
-		 (int)isc_buffer_availablelength(&httpd->headerbuffer),
-		 "%s %03u %s\r\n", httpd->protocol, httpd->retcode,
-		 httpd->retmsg);
-	isc_buffer_add(&httpd->headerbuffer, needlen);
-
-	return (ISC_R_SUCCESS);
+	return (isc_buffer_printf(&httpd->headerbuffer, "%s %03u %s\r\n",
+				   httpd->protocol, httpd->retcode,
+				   httpd->retmsg));
 }
 
 isc_result_t
@@ -943,18 +1073,13 @@ isc_httpd_addheader(isc_httpd_t *httpd, const char *name,
 			return (result);
 	}
 
-	if (val != NULL)
-		snprintf(isc_buffer_used(&httpd->headerbuffer),
-			 isc_buffer_availablelength(&httpd->headerbuffer),
-			 "%s: %s\r\n", name, val);
-	else
-		snprintf(isc_buffer_used(&httpd->headerbuffer),
-			 isc_buffer_availablelength(&httpd->headerbuffer),
-			 "%s\r\n", name);
-
-	isc_buffer_add(&httpd->headerbuffer, needlen);
-
-	return (ISC_R_SUCCESS);
+	if (val != NULL) {
+		return (isc_buffer_printf(&httpd->headerbuffer, "%s: %s\r\n",
+					   name, val));
+	} else {
+		return (isc_buffer_printf(&httpd->headerbuffer, "%s\r\n",
+					   name));
+	}
 }
 
 isc_result_t
@@ -967,11 +1092,7 @@ isc_httpd_endheaders(isc_httpd_t *httpd) {
 			return (result);
 	}
 
-	snprintf(isc_buffer_used(&httpd->headerbuffer),
-		 isc_buffer_availablelength(&httpd->headerbuffer), "\r\n");
-	isc_buffer_add(&httpd->headerbuffer, 2);
-
-	return (ISC_R_SUCCESS);
+	return (isc_buffer_printf(&httpd->headerbuffer, "\r\n"));
 }
 
 isc_result_t
@@ -992,13 +1113,8 @@ isc_httpd_addheaderuint(isc_httpd_t *httpd, const char *name, int val) {
 			return (result);
 	}
 
-	snprintf(isc_buffer_used(&httpd->headerbuffer),
-		 isc_buffer_availablelength(&httpd->headerbuffer),
-		 "%s: %s\r\n", name, buf);
-
-	isc_buffer_add(&httpd->headerbuffer, needlen);
-
-	return (ISC_R_SUCCESS);
+	return (isc_buffer_printf(&httpd->headerbuffer, "%s: %s\r\n", name,
+				   buf));
 }
 
 static void
@@ -1027,14 +1143,18 @@ isc_httpd_senddone(isc_task_t *task, isc_event_t *ev) {
 	 */
 	if (httpd->freecb != NULL) {
 		isc_buffer_t *b = NULL;
-		if (isc_buffer_length(&httpd->bodybuffer) > 0)
+		if (isc_buffer_length(&httpd->bodybuffer) > 0) {
 			b = &httpd->bodybuffer;
-		httpd->freecb(b, httpd->freecb_arg);
+			httpd->freecb(b, httpd->freecb_arg);
+		}
 		NOTICE("senddone free callback performed");
 	}
 	if (ISC_LINK_LINKED(&httpd->bodybuffer, link)) {
 		ISC_LIST_UNLINK(sev->bufferlist, &httpd->bodybuffer, link);
 		NOTICE("senddone body buffer unlinked");
+	} else if (ISC_LINK_LINKED(&httpd->compbuffer, link)) {
+		ISC_LIST_UNLINK(sev->bufferlist, &httpd->compbuffer, link);
+		NOTICE("senddone compressed data unlinked and freed");
 	}
 
 	if (sev->result != ISC_R_SUCCESS) {
@@ -1084,6 +1204,7 @@ reset_client(isc_httpd_t *httpd) {
 	httpd->flags = 0;
 
 	isc_buffer_clear(&httpd->headerbuffer);
+	isc_buffer_clear(&httpd->compbuffer);
 	isc_buffer_invalidate(&httpd->bodybuffer);
 }
 
@@ -1091,12 +1212,12 @@ isc_result_t
 isc_httpdmgr_addurl(isc_httpdmgr_t *httpdmgr, const char *url,
 		    isc_httpdaction_t *func, void *arg)
 {
-	return (isc_httpdmgr_addurl2(httpdmgr, url, ISC_FALSE, func, arg));
+	return (isc_httpdmgr_addurl2(httpdmgr, url, false, func, arg));
 }
 
 isc_result_t
 isc_httpdmgr_addurl2(isc_httpdmgr_t *httpdmgr, const char *url,
-		     isc_boolean_t isstatic,
+		     bool isstatic,
 		     isc_httpdaction_t *func, void *arg)
 {
 	isc_httpdurl_t *item;
@@ -1126,3 +1247,9 @@ isc_httpdmgr_addurl2(isc_httpdmgr_t *httpdmgr, const char *url,
 
 	return (ISC_R_SUCCESS);
 }
+
+void
+isc_httpd_setfinishhook(void (*fn)(void))
+{
+	finishhook = fn;
+}
